﻿using System;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Text.RegularExpressions;

namespace RinGOch.HtmlGenerators
{
	public class HtmlTool
	{
		#region 利用可能なエレメント名＆スキーム
		static Hashtable VaildElements;
		static Hashtable VaildUriScheme;
		static HtmlTool()
		{
			foreach(char c in System.IO.Path.InvalidPathChars)
				_InvalidPathPattern += Regex.Escape(c.ToString()) + "|";
			_InvalidPathPattern = string.Concat("(",Regex.Escape("../"),"|",_InvalidPathPattern.TrimEnd('|'),")");

			VaildUriScheme = new Hashtable();
			VaildUriScheme.Add("src",null);
			VaildUriScheme.Add("href",null);
			VaildUriScheme.Add("cite",null);

			VaildElements = new Hashtable();
			VaildElements.Add("a",new ValidElementDadta("a","href","name","target"));
			VaildElements.Add("abbr",new ValidElementDadta("abbr"));
			VaildElements.Add("acronym",new ValidElementDadta("acronym"));
			VaildElements.Add("address",new ValidElementDadta("address"));
			VaildElements.Add("b",new ValidElementDadta("b"));
			VaildElements.Add("basefont",new ValidElementDadta("basefont","face"));
			VaildElements.Add("big",new ValidElementDadta("big"));
			VaildElements.Add("blockquote",new ValidElementDadta("blockquote","cite"));
			VaildElements.Add("body",new ValidElementDadta("body"));
			VaildElements.Add("br",new ValidElementDadta("br","clear"));
			VaildElements.Add("col",new ValidElementDadta("col","span"));
			VaildElements.Add("dfn",new ValidElementDadta("dfn"));
			VaildElements.Add("em",new ValidElementDadta("em"));
			VaildElements.Add("fieldset",new ValidElementDadta("fieldset"));
			VaildElements.Add("caption",new ValidElementDadta("caption"));
			VaildElements.Add("center",new ValidElementDadta("center"));
			VaildElements.Add("cite",new ValidElementDadta("cite"));
			VaildElements.Add("code",new ValidElementDadta("code"));
			VaildElements.Add("div",new ValidElementDadta("div"));
			VaildElements.Add("del",new ValidElementDadta("del"));
			VaildElements.Add("dl",new ValidElementDadta("dl"));
			VaildElements.Add("dd",new ValidElementDadta("dd"));
			VaildElements.Add("dt",new ValidElementDadta("dt"));
			VaildElements.Add("html",new ValidElementDadta("html","xmlns","lang"));
			VaildElements.Add("head",new ValidElementDadta("head"));
			VaildElements.Add("font",new ValidElementDadta("font","face"));
			VaildElements.Add("form",new ValidElementDadta("form","action","method","target","enctype"));
			VaildElements.Add("h1",new ValidElementDadta("h1"));
			VaildElements.Add("h2",new ValidElementDadta("h2"));
			VaildElements.Add("h3",new ValidElementDadta("h3"));
			VaildElements.Add("h4",new ValidElementDadta("h4"));
			VaildElements.Add("h5",new ValidElementDadta("h5"));
			VaildElements.Add("h6",new ValidElementDadta("h6"));
			VaildElements.Add("hr",new ValidElementDadta("hr"));
			VaildElements.Add("i",new ValidElementDadta("i"));
			VaildElements.Add("img",new ValidElementDadta("img","src"));
			VaildElements.Add("input",new ValidElementDadta("input","type","name","value","tabindex","checked"));
			VaildElements.Add("ins",new ValidElementDadta("ins"));
			VaildElements.Add("kbd",new ValidElementDadta("kbd"));
			VaildElements.Add("label",new ValidElementDadta("label","for"));
			VaildElements.Add("legend",new ValidElementDadta("legend"));
			VaildElements.Add("li",new ValidElementDadta("li"));
			VaildElements.Add("link",new ValidElementDadta("link","type","rel","href"));
			VaildElements.Add("meta",new ValidElementDadta("meta","name","content","http-equiv"));
			VaildElements.Add("ol",new ValidElementDadta("ol"));
			VaildElements.Add("optgroup",new ValidElementDadta("optgroup","label"));
			VaildElements.Add("option",new ValidElementDadta("option","value"));
			VaildElements.Add("p",new ValidElementDadta("p"));
			VaildElements.Add("pre",new ValidElementDadta("pre"));
			VaildElements.Add("q",new ValidElementDadta("q"));
			VaildElements.Add("s",new ValidElementDadta("s"));
			VaildElements.Add("samp",new ValidElementDadta("samp"));
			VaildElements.Add("select",new ValidElementDadta("select","name","accesskey","tabindex"));
			VaildElements.Add("small",new ValidElementDadta("small"));
			VaildElements.Add("span",new ValidElementDadta("span"));
			VaildElements.Add("strike",new ValidElementDadta("strike"));
			VaildElements.Add("strong",new ValidElementDadta("strong"));
			VaildElements.Add("sub",new ValidElementDadta("sub"));
			VaildElements.Add("sup",new ValidElementDadta("sup"));
			VaildElements.Add("table",new ValidElementDadta("table","cellpadding","cellspacing"));
			VaildElements.Add("tbody",new ValidElementDadta("tbody"));
			VaildElements.Add("td",new ValidElementDadta("td","rowspan","colspan"));
			VaildElements.Add("textarea",new ValidElementDadta("textarea","name","cols","rows"));
			VaildElements.Add("tfoot",new ValidElementDadta("tfoot"));
			VaildElements.Add("th",new ValidElementDadta("th","rowspan","colspan"));
			VaildElements.Add("thead",new ValidElementDadta("thead"));
			VaildElements.Add("title",new ValidElementDadta("title"));
			VaildElements.Add("tr",new ValidElementDadta("tr"));
			VaildElements.Add("tt",new ValidElementDadta("tt"));
			VaildElements.Add("u",new ValidElementDadta("u"));
			VaildElements.Add("ul",new ValidElementDadta("ul"));
			VaildElements.Add("var",new ValidElementDadta("var"));
		}
		#endregion

		#region 利用不能なURI-CharのRegularExpressionsPattern
		static string _InvalidPathPattern;
		#endregion

		#region 有効エレメント構造体
		class ValidElementDadta
		{
			public string ElementName;
			public string[] Attributes;
			public static readonly string[] AlwayValidAttributes = new string[]{"align","alt","background","bgcolor","border","cite","class","datetime","height","id","size","style","title","type","valign","width"};

			public ValidElementDadta(string element,params string[] attri)
			{
				ElementName = element;
				Attributes = attri;

				// 属性＆エレメントを小文字にする
				ElementName = ElementName.ToLower();
				for(int i=0;i<Attributes.Length;i++)
					Attributes[i] = Attributes[i].ToLower();
			}

			/// <summary>
			/// 引数に指定された属性が有効か調べます
			/// </summary>
			public bool IsVaildAttribute(string att)
			{
				string lowerAtt = att.ToLower();
				
				if(Array.IndexOf(this.Attributes,lowerAtt) >= 0) return true;
				if(Array.IndexOf(ValidElementDadta.AlwayValidAttributes,lowerAtt) >= 0) return true;

				return false;
			}
		}
		#endregion

		#region 危険HTMLを無効化
		public static string CancelDangerTag(string html)
		{
			return Regex.Replace(html,
				@"\<(((?!(%|!))\s*(?<slash>(/|))\s*(?<ele>[^\>\s/]+)(?<att>[^\>]*?))|((?<ele>![^\>]*)))\>"
				,new MatchEvaluator(MatchBracketEvaluator),RegexOptions.Singleline);
		}
		static string MatchBracketEvaluator(Match match)
		{
			string element = match.Groups["ele"].Value.Trim();
			string allatt = (match.Groups["att"].Success ? match.Groups["att"].Value.Trim() : "");
			string retatt = "";

			// 有効なエレメント名かチェック
			ValidElementDadta ve = VaildElements[element.ToLower()] as ValidElementDadta;
			if(ve == null)
			{
				if(element.StartsWith("!"))
				{
					return string.Concat("<",element,">");
				}
				else
				{
					// 無効なエレメント名なので、HTMLEncodingして返す
					return HttpUtility.HtmlEncode(match.Value);
				}
			}
			else
			{
				// 属性を分解します
				MatchCollection atts = Regex.Matches(allatt,"(((?<name>[^=]+)=\\s*(?<value>(\"[^\"]*\"|[^\\s]*)))|\\s*(?<slash>/)\\s*)",RegexOptions.Singleline);

				// 有効な属性なら追加します
				foreach(Match att in atts)
				{
					if(att.Groups["slash"].Success)
					{
						// XHTMLのエレメント終端スラッシュなので、結果をリターン
						return string.Concat("<",element," ",retatt,"/>");
					}
					else
					{
						string strAttName = ( att.Groups["name"].Success ? att.Groups["name"].Value.Trim() : "");
						string strAttValue = ( att.Groups["value"].Success ? att.Groups["value"].Value.Trim().Trim('\"') : "");

						// 有効な属性かチェック
						if(ve.IsVaildAttribute(strAttName.ToLower()) && IsValidUriScheme(strAttName.ToLower(),strAttValue.ToLower()))
						{
							// スタイルシートならチェック
							CheckStyle(strAttName,ref strAttValue);

							// 追加
							retatt = string.Concat(retatt,strAttName.ToLower(),"=\"",strAttValue,"\" ");
						}
					}
				}
			}

			return string.Concat("<",match.Groups["slash"].Value,element,(retatt=="" ? "" : " "),retatt.Trim(),">");
		}
		static void CheckStyle(string attName,ref string attValue)
		{
			if(attName.ToLower() == "style")
			{
				attValue = CancelDangerCSS(attValue);
			}
		}
		static bool IsValidUriScheme(string url)
		{
			if(Regex.IsMatch(url,_InvalidPathPattern))
			{
				return false;
			}
			else
			{
				if(url.StartsWith("http://") || url.StartsWith("https://") || url.StartsWith("ftp://"))
					return true;
				else if(url.StartsWith("files/"))
					return true;
				else if(url.StartsWith("&") || url.StartsWith("%"))
					return true;
				else
					return false;
			}
		}
		static bool IsValidUriScheme(string attName,string attValue)
		{
			if(VaildUriScheme.ContainsKey(attName))
			{
				return IsValidUriScheme(attValue);
			}
			else
				return true;
		}
		#endregion

		#region 危険CSSを無効化
		public static string CancelDangerCSS(string css)
		{
			// 置き換え-1
			css = Regex.Replace(css,"\\\\[^'\"\\{};:\\(\\)\\#A\\*]{1}","",RegexOptions.IgnoreCase | RegexOptions.Singleline);

			// 置き換え-2
			css = Regex.Replace(css,"javascript|vbscript|@import|cookie|eval|expression|behavior|behaviour|binding|include-source|@i","",RegexOptions.IgnoreCase | RegexOptions.Singleline);

			// URLチェック
			css = Regex.Replace(css,"url\\s*\\((\"(?<url>[^\"]*)\"\\s*\\)|(?<url>[^\\)]*)\\))",new MatchEvaluator(MatchCssUrlEvaluator),RegexOptions.IgnoreCase | RegexOptions.Singleline);

			// 最終置き換え
			css = css.Replace("<","&lt;").Replace(">","&gt;");

			return css;
		}
		static string MatchCssUrlEvaluator(Match match)
		{
			string url = (match.Groups["url"].Success ? match.Groups["url"].Value.Trim() : "");
			if(IsValidUriScheme(url.ToLower()))
				return string.Concat("url(\"",url,"\")");
			else
				return "url()";
		}
		#endregion

		#region クッキー
		public static string GetCookieValue(string cookie,string name)
		{
			if(cookie==null) cookie ="";
			Match m = Regex.Match(cookie,string.Concat(name,"=","(?<value>[^;]*)"));
			if(m.Groups["value"].Success)
				return m.Groups["value"].Value;
			else
				return "";
		}
		#endregion
	}
}
