﻿using System;
using System.Threading;
using System.Reflection;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Text.RegularExpressions;
using System.Security.Cryptography;
using System.Diagnostics;

namespace RinGOch.HtmlGenerators
{
	public class MenuGenerator
	{
		#region 分岐
		public static byte[] Generate(StringDictionary param,ref string mime)
		{
			try
			{
				string path = param["_filename"].ToLower();
				if(path.StartsWith("/"))path=path.Substring(1);
				switch(path)
				{
					case "search.p2p":
						return HttpServerSetting.HtmlEncoding.GetBytes(SearchPage(param));
					case "ignore.p2p":
						return PostProcessors.ManagementProcessor.ProcessIgnore(param);
					case "":
					case "index.html":
						return WebIndexPage(param);
					case "top.html":
						return WebTopPage(param);
					case "menu.html":
						return WebMenuPage(param);
					case "default.css":
						return HttpServerSetting.HtmlEncoding.GetBytes(HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\default.css"));
					case "list.html":
						return HttpServerSetting.HtmlEncoding.GetBytes(WebListPage(param,false));
					case "index.rdf":
						return HttpServerSetting.HtmlEncoding.GetBytes(WebListPage(param,true));
					case "new.html":
						return WebNewPage(param);
					case "certification.png":
						return WebImageCertification();
					case "trip.p2p":
						return TripGeneratorPage(param);
					case "chat.p2p":
						return ChatGenerator.Generate(param);
					case "export.p2p":
						return ExportPage(param);
					case "cachetools.p2p":
						return HttpServerSetting.HtmlEncoding.GetBytes(ProcessCacheTool(param));
					case "bbsmenu.html":
						return Encoding.GetEncoding("shift-jis").GetBytes(TwoChGenerator.GenerateMenu());
					case "manage_wiki.p2p":
					case "manage.p2p":
						return ManageGenerator.Generate(param);
					case "blogmgr.p2p":
						return BlogEngine.GenerateManagePage(param,ref mime);
					case "threadinfo.p2p":
						return HttpServerSetting.HtmlEncoding.GetBytes(GetThreadInfo(param,ref mime));
					default:
					{
						path = param["_path"].ToLower();
						if(path.StartsWith("/"))path=path.Substring(1);

						if(Is2chUri(path))
						{
							mime = "text/plain";
							path = HttpUtility.UrlDecode(path);
							string[] a = path.Split('/');
							Encoding enc = Encoding.GetEncoding("shift-jis");
							if(path.ToLower().EndsWith(".dat"))
							{
								return enc.GetBytes(TwoChGenerator.GenerateDat(new Guid(a[2].Substring(0,a[2].Length-4))));
							}
							else if(path.ToLower().EndsWith("subject.txt"))
							{
								return enc.GetBytes(TwoChGenerator.GenerateSubject(a[0]));
							}
							else
								throw new Exception("不正なURIが渡されました。2ch互換DAT出力時のエラーです");
						}
						else
						{
							string p2 = Path.Combine(System.Environment.CurrentDirectory,"template\\gateway\\" + path);

							// RinGOchの脆弱性：http://localhost/../../../　でローカルファイルにアクセスできる問題
							if(p2.IndexOf("..") >= 0 || !p2.StartsWith(Path.Combine(System.Environment.CurrentDirectory,"template\\gateway\\")))
							{
								mime = "text/html";
								return ErrorHtml.Generate("指定されたファイルが見つかりません。",null);
							}

							if(File.Exists(p2))
							{
								using(FileStream fs = new FileStream(p2,FileMode.Open,FileAccess.Read,FileShare.Read))
								{
									byte[] buffer = new byte[fs.Length];
									fs.Read(buffer,0,buffer.Length);
									fs.Close();
									return buffer;
								}
							}
							else
							{
								p2 = Path.Combine(System.Environment.CurrentDirectory,"template\\xml\\" + path);
								if(File.Exists(p2))
								{
									if(Path.GetExtension(p2).ToLower() == ".xml")
									{
										StringBuilder temp = new StringBuilder(HttpHtmlGenerator.ReadTemplateFile("template\\xml\\" + path));
										temp.Replace("&listXML;",XmlGenerator.GenerateThreadListXML());
										return HttpServerSetting.HtmlEncoding.GetBytes(temp.ToString());
									}
									else
									{
										using(FileStream fs = new FileStream(p2,FileMode.Open,FileAccess.Read,FileShare.Read))
										{
											byte[] buffer = new byte[fs.Length];
											fs.Read(buffer,0,buffer.Length);
											fs.Close();
											return buffer;
										}
									}
								}
								else
								{
									mime = "text/html";
									return ErrorHtml.Generate("指定されたファイルが見つかりません。",null);
								}
							}
						}
					}
				}
			}
			catch(HttpException)
			{
				throw;
			}
			catch(Exception ex)
			{				
				mime = "text/html";
				return ErrorHtml.Generate("ゲートウェイのHTMLを生成中に例外が発生しました",ex);
			}
		}
		static bool Is2chUri(string path)
		{
			path = HttpUtility.UrlDecode(path);
			path = path.Replace("\\","/");
			Debug.WriteLine("Path:" + path);
			string[] a = path.Split('/');
			if(a.Length == 0) return false;
			bool flag = false;
			if(a[0] == "すべて")
				flag = true;
			else
			{
				foreach(string cate in WebCategories.Categories)
				{
					if(cate.ToLower() == a[0])
					{
						flag = true;
						break;
					}
				}
			}
			if(!flag)
			{
				return false;
			}

			if(a.Length == 3)
			{
				if((a[1].ToLower() == "dat" && a[2].ToLower().EndsWith(".dat")))
				{
					try
					{
						Guid id = new Guid(a[2].Substring(0,a[2].Length-4));
						return true;
					}
					catch
					{
						return false;
					}
				}
			}
			else if(a.Length == 2)
			{
				if(a[1].ToLower() == "subject.txt") return true;
			}
			
			return false;
		}
		#endregion

		#region index.html
		static byte[] WebIndexPage(StringDictionary param)
		{
			return HttpServerSetting.HtmlEncoding.GetBytes(HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\index.html"));
		}
		#endregion

		#region menu.html
		static byte[] WebMenuPage(StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\menu.html");
			string loop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN LOOP-->","<!--END LOOP-->");
			html.Append(temp.Substring(0,temp.IndexOf("<!--BEGIN LOOP-->")));
			int[] counter = new int[WebCategories.Categories.Length];
			object[] headers = Interface.GetWebHeaders();

			try
			{
				foreach(WebHeaderInfo info in headers)
				{
					try
					{
						int q = Array.IndexOf(WebCategories.Categories,info.Category);
						if(q >= 0)counter[q]++;
					}
					catch{}
				}
			}
			catch{}

			html.Append(loop.Replace("%ENCCATEGORY%","").Replace("%CATEGORY%","すべて").Replace("%ITEMCOUNT%",headers.Length.ToString()));
			foreach(string cate in WebCategories.Categories)
			{
				html.Append(loop.Replace("%ENCCATEGORY%",System.Web.HttpUtility.UrlEncode(cate)).Replace("%CATEGORY%",cate).Replace("%ITEMCOUNT%",(counter[Array.IndexOf(WebCategories.Categories,cate)]).ToString()));
			}

			html.Append(temp.Substring(temp.IndexOf("<!--END LOOP-->")+"<!--END LOOP-->".Length));
			return HttpServerSetting.HtmlEncoding.GetBytes(html.ToString());
		}
		#endregion

		#region top.html
		static string WebTopPageReplaceInfo(string temp)
		{
			System.Diagnostics.Process proc = System.Diagnostics.Process.GetCurrentProcess();
			try
			{
				temp = temp.Replace("%_ringoch_mem_manage%",TextUtility.SizeToString(GC.GetTotalMemory(true)));
			}
			catch
			{
				temp = temp.Replace("%_ringoch_mem_managed%","ERROR");
			}

			try
			{
				temp = temp.Replace("%_p2p_mem_workingset%",TextUtility.SizeToString(proc.WorkingSet));
			}
			catch
			{
				temp = temp.Replace("%_p2p_mem_workingset%","ERROR");
			}

			try
			{
				temp = temp.Replace("%_p2p_mem_virtual%",TextUtility.SizeToString(proc.VirtualMemorySize));
			}
			catch
			{
				temp = temp.Replace("%_p2p_mem_virtual%","ERROR");
			}

			try
			{
				TimeSpan span = DateTime.Now.Subtract(proc.StartTime);
				temp = temp.Replace("%_ringoch_info_time%",span.Hours.ToString("00") + ":" + span.Minutes.ToString("00") + ":" + span.Seconds.ToString("00"));
			}
			catch
			{
				temp = temp.Replace("%_ringoch_info_time%","ERROR");
			}

			try
			{
				temp = temp.Replace("%_ringoch_info_active_connections%",HttpServer.ActiveConnections.ToString());
			}
			catch
			{
				temp = temp.Replace("%_ringoch_info_active_connections%","ERROR");
			}

			Version ringoVer = Assembly.GetExecutingAssembly().GetName().Version;
			temp = temp.Replace("%_ringoch_ver%",EngineVersion.RinGOchVersion);
			temp = temp.Replace("%_blog_ver%",EngineVersion.BlogEngineVersion);
			temp = temp.Replace("%_wiki_ver%",EngineVersion.WikiEngineVersion);

			return temp;
		}
		static byte[] WebTopPage(StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\top.html");

			string ver;
			int cons;
			Interface.GetInformation(out ver,out cons);
			temp = temp
				.Replace("%_p2p_version%",ver)
				.Replace("%_p2p_connections%",cons.ToString());
			temp = WebTopPageReplaceInfo(temp);

			html.Append(temp.Substring(0,temp.IndexOf("<!--BEGIN PROGRAMMABLE-->")));
			Guid[] updated = WebPageUpdateFlag.GetUpdatedPages();
			if(updated.Length > 0)
			{
				string loop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN LOOP-->","<!--END LOOP-->");
				string bbsLatest = "";
				foreach(Guid id in updated)
				{
					WebHeaderInfo info = WebCacheReader.ReadHeader(id);

					if(info != null)
					{
						if((info.Type==WebType.FileBoard || info.Type == WebType.ImageBoard || info.Type == WebType.TextOnlyBoard))
							bbsLatest = HttpServerSetting.BBSParameter.Replace("/","");
						else
							bbsLatest = "";
						html.Append(TemplateFieldReplacer.ReplaceGatewayFields(loop,info,"",0).Replace("%_bbsLatest%",bbsLatest));
					}
				}
			}
			else
			{
				html.Append(TextUtility.GetMiddleRangeString(temp,"<!--BEGIN NO UPDATED-->","<!--END NO UPDATED-->"));
			}

			html.Append(temp.Substring(temp.IndexOf("<!--END PROGRAMMABLE-->")+"<!--END PROGRAMMABLE-->".Length));
			return HttpServerSetting.HtmlEncoding.GetBytes(html.ToString());
		}
		#endregion

		#region list.html
		static string WebListPage(StringDictionary param,bool IsTwoLoop)
		{
			StringBuilder html = new StringBuilder();
			int index = 0;
			string temp;
			if(IsTwoLoop)
				temp = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\index.rdf");
			else
				temp = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\list.html");
			if(IsTwoLoop)
			{
				temp = HttpHtmlGenerator.ReplaceLocalAddress(temp);
				temp = temp.Replace("&utcNow;",DateTime.UtcNow.ToString("yyyy-MM-dd hh\\:mm\\:ss"));
			}
			string cate = param["cate"];
			bool flag = true;
			object[] headers = Interface.GetWebHeaders();
			int max = (IsTwoLoop ? 50 : headers.Length);

			try
			{
				flag = bool.Parse(param["order"]);
			}
			catch{}

			if(cate == null)cate="";
			if(cate == "すべて")cate="";

			ArrayList list = new ArrayList();

			// ソート
			IComparer ic = null;
			switch(param["sort"])
			{
				default:
				case null:
				case "new":
					ic = new WebHeaderComparer(WebHeaderComparerField.Update,flag);
					break;
				case "title":
					ic = new WebHeaderComparer(WebHeaderComparerField.Title,!flag);
					break;
				case "guid":
					ic = new WebHeaderComparer(WebHeaderComparerField.Guid,!flag);
					break;
				case "posts":
					ic = new WebHeaderComparer(WebHeaderComparerField.Posts,flag);
					break;
				case "size":
					ic = new WebHeaderComparer(WebHeaderComparerField.Size,flag);
					break;
				case "mgr":
					ic = new WebHeaderComparer(WebHeaderComparerField.MgrUpdate,flag);
					break;
				case "type":
					ic = new WebHeaderComparer(WebHeaderComparerField.Type,!flag);
					break;
				case "ctg":
					ic = new WebHeaderComparer(WebHeaderComparerField.Category,!flag);
					break;
			}
			Array.Sort(headers,ic);

			DateTime latestDate = DateTime.MinValue;

			try
			{
				for(int i=0;i<headers.Length;i++)
				{
					try
					{
						WebHeaderInfo info = (WebHeaderInfo)headers[i];

						if(
							(cate == "" || info.Category.ToLower()==cate.ToLower()) &&
							(param["fw"]==null || param["fw"]=="" || info.Title.ToLower().IndexOf(param["fw"].ToLower())>=0) &&
							(IntegerParse(param["ft"])<-3 || IntegerParse(param["ft"])==-2 || 
							(IntegerParse(param["ft"])==-1 && (info.Type==WebType.TextOnlyBoard || info.Type==WebType.ImageBoard || info.Type==WebType.FileBoard)) ||
							(IntegerParse(param["ft"]) == ((int)info.Type))
							) &&
							(IntegerParse(param["fu"])<=0 || DateTime.Now.Subtract(info.Update).TotalHours <=IntegerParse(param["fu"])) &&
							(IntegerParse(param["fm"])<=0 || DateTime.Now.Subtract(info.Created).TotalHours <=IntegerParse(param["fm"]))
							)
						{
							if(WebCacheReader.IsCached(info.GUID))
								list.Add(WebCacheReader.ReadHeader(info.GUID));
							else
								list.Add(info);
							if (latestDate < info.Update)
								latestDate = info.Update;
							if(list.Count >= max) break;
						}
					}
					catch{}
				}
			}
			catch{}

			string strTemp = temp.Substring(0,temp.IndexOf("<!--BEGIN LOOP-->"));
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%CATEGORY%",(cate==""?"すべて":cate));
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%ORDER%",(!flag).ToString());
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%!ORDER%",(flag).ToString());
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%SORT%",param["sort"]);
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%FTYPE%",param["ft"]);
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%FT-" + param["ft"] + "%","selected");
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%FUPDATE%",param["fu"]);
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%FU-" + param["fu"] + "%","selected");
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%FMANAGE%",param["fm"]);
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%FM-" + param["fm"] + "%","selected");
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%FWORD%",param["fw"]);
			strTemp = TemplateFieldReplacer.ReplaceField(strTemp,"%LATESTUPDATE%",latestDate.ToString ("yyyy'-'MM'-'dd'T'HH':'mm':'sszzz"));
			html.Append(strTemp);

			for(int i=0;i<2;i++)
			{
				string loop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN LOOP-->","<!--END LOOP-->");
				string bbsLatest = "";

				foreach(WebHeaderInfo info in list)
				{
					string st = "";
					if(info != null)
					{
						if(WebCacheReader.IsCached(info.GUID))
						{
							if(WebPageUpdateFlag.GetFlag(info.GUID,false))
								st = "updated";
							else
								st = "read";
						}
						else
						{
							if(DateTime.Now.Subtract(info.Update).TotalHours <= 1)
								st = "updated-nonread";
							else
								st = "nonread";
						}
						if((info.Type==WebType.FileBoard || info.Type == WebType.ImageBoard || info.Type == WebType.TextOnlyBoard))
							bbsLatest = HttpServerSetting.BBSParameter.Replace("/","");
						else
							bbsLatest = "";
						html.Append(TemplateFieldReplacer.ReplaceGatewayFields(loop,info,st,++index).Replace("%_bbsLatest%",bbsLatest));
					}
				}
				html.Append(temp.Substring(temp.IndexOf("<!--END LOOP-->")+"<!--END LOOP-->".Length));

				if(!IsTwoLoop) break;
				if(IsTwoLoop && i==1) break;

				temp = html.ToString();
				html = new StringBuilder();
				html.Append(temp.Substring(0,temp.IndexOf("<!--BEGIN LOOP-->")));
			}
			return html.ToString();
		}
		#endregion

		#region new.html
		public const int LIMITMINUTES = 5;
		static byte[] WebNewPage(StringDictionary param)
		{
			if(!HttpServer.HasAuthority(param))
			{
				throw HttpException.UnauthorizedException;
			}
			else
			{
				string temp = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\new.html");
				int remain = 0;
				if(Interface.CanCreatePage(out remain))
				{
					// LIMITエリアを削除
					temp = temp.Replace(TextUtility.GetMiddleRangeString(temp,"<!--BEGIN LIMIT-->","<!--END LIMIT-->"),"");
					string cate = "";
					foreach(string c in WebCategories.Categories)
						cate +="<option>" + c;
					temp = temp.Replace("%CATEGORIES%",cate);
				}
				else
				{
					// OKエリアを削除
					temp = temp.Replace(TextUtility.GetMiddleRangeString(temp,"<!--BEGIN OK-->","<!--END OK-->"),"");
					temp = temp.Replace("%REMAIN%",remain.ToString() + "分");
				}
				return HttpServerSetting.HtmlEncoding.GetBytes(temp);
			}
		}
		static byte[] WebImageCertification()
		{
			System.Drawing.Bitmap bmp = Interface.GenerateCertificationImage();
			using(MemoryStream ms = new MemoryStream())
			{
				bmp.Save(ms,System.Drawing.Imaging.ImageFormat.Png);
				return ms.GetBuffer();
			}
		}
		static int IntegerParse(string data)
		{
			try
			{
				return int.Parse(data);
			}
			catch
			{
				return int.MinValue;
			}
		}
		#endregion

		#region trip.p2p
		static byte[] TripGeneratorPage(StringDictionary param)
		{
			if(!HttpServer.HasAuthority(param))
			{
				throw HttpException.UnauthorizedException;
			}
			else
			{
				string temp = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\trip.html");
				using(RSACryptoServiceProvider rsa = new RSACryptoServiceProvider(384))
				{
					temp = temp.Replace("%_privateKey%",HttpUtility.HtmlEncode(rsa.ToXmlString(true)));
					temp = temp.Replace("%_trip%",TextUtility.XmlToTrip(rsa.ToXmlString(false)));
					rsa.Clear();
				}
				return HttpServerSetting.HtmlEncoding.GetBytes(temp);
			}
		}
		#endregion

		#region export.p2p
		static byte[] ExportPage(StringDictionary param)
		{
			if(param.ContainsKey("guid"))
			{
				try
				{
					Guid id = new Guid(param["guid"]);
					WebHeaderInfo header = WebCacheReader.ReadHeader(id);
					WebPostInfo[] posts = WebCacheReader.ReadAllPost(id);

					if(param.ContainsKey("name"))
					{
						return HttpServerSetting.HtmlEncoding.GetBytes(WikiGenerator.GetNewestWikiName(posts,header,param["name"],Guid.Empty).Body);
					}
					else if(param.ContainsKey("post"))
					{
						Guid res = new Guid(param["post"]);
						foreach(WebPostInfo post in posts)
						{
							if(post.GUID.Equals(res))
							{
								return HttpServerSetting.HtmlEncoding.GetBytes(post.Body);
							}
						}
						return new byte[0];
					}
					else
					{
						return new byte[0];
					}
				}
				catch(Exception ex)
				{
					return ErrorHtml.Generate("エクスポート処理中に例外が発生しました。パラメータが不正な可能性があります",ex);
				}
			}
			else
				return new byte[0];
		}
		#endregion

		#region search.p2p
		static string _local = HttpHtmlGenerator.CreateLocalAddress();
		static string ReplaceChecked(string temp,string id,bool check)
		{
			if(check)
				return temp.Replace(id,"checked");
			else
				return temp.Replace(id,"");
		}
		static string SearchPage(StringDictionary param)
		{
			if(HttpServer.HasAuthority(param))
			{
				if(param.ContainsKey("search"))
				{
					bool searchTextBBS = param.ContainsKey("textbbs");
					bool searchFileBBS = param.ContainsKey("filebbs");
					bool searchImageBBS = param.ContainsKey("imagebbs");
					bool searchWiki = param.ContainsKey("wiki");
					bool searchBlog = param.ContainsKey("blog");

					DateTime start = DateTime.Now;
					const string loop_key = "<!--LOOP-->";
					string temp_base = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\search_result.html");
					string loop = TextUtility.GetMiddleRangeString(temp_base,loop_key,loop_key);
					StringBuilder temp = new StringBuilder();
					object[] headers = Interface.GetWebHeaders();
					Regex regex = null;
					try
					{
						regex = new Regex(string.Concat("(?<summary>.{0,30}",param["search"],".{0,30})"),RegexOptions.IgnoreCase | RegexOptions.Singleline | RegexOptions.Compiled);
					}
					catch
					{
						return temp_base.Replace(string.Concat(loop_key,loop,loop_key),"不正な正規表現です。検索を実行できません");
					}

					bool errorFlag = false;
					int hitThreads = 0;
					int hitRes = 0;
					int totalThreads = 0;
					int totalRes = 0;
					foreach(WebHeaderInfo info in headers)
					{
						try
						{
							if(WebCacheReader.IsCached(info))
							{
								totalThreads++;
								if((searchTextBBS && info.Type == WebType.TextOnlyBoard) || (searchImageBBS && info.Type == WebType.ImageBoard) || (searchFileBBS && info.Type == WebType.FileBoard))
								{
									SearchBBSPage(info,regex,loop,ref temp,ref hitThreads,ref totalRes,ref hitRes);
								}
								else if(searchBlog && info.Type == WebType.Blog)
								{
									BlogEngine.Search(info,regex,loop,ref temp,_local,true,ref hitThreads,ref totalRes,ref hitRes);
								}
								else if(searchWiki && info.Type == WebType.Wiki)
								{
									SearchWikiPage(info,regex,loop,ref temp,ref hitThreads,ref totalRes,ref hitRes);
								}
							}
						}
						catch
						{
							errorFlag = true;
						}
					}
					temp_base = ReplaceChecked(temp_base,"%check_text%",searchTextBBS);
					temp_base = ReplaceChecked(temp_base,"%check_image%",searchImageBBS);
					temp_base = ReplaceChecked(temp_base,"%check_file%",searchFileBBS);
					temp_base = ReplaceChecked(temp_base,"%check_wiki%",searchWiki);
					temp_base = ReplaceChecked(temp_base,"%check_blog%",searchBlog);

					if(errorFlag)
						temp_base = temp_base.Replace("&error;","検索中にエラーが発生しました。破損キャッシュ自動削除ツールを試してみてください");
					else
						temp_base = temp_base.Replace("&error;","");

					temp_base = temp_base.Replace("&totalThreads;",totalThreads.ToString());
					temp_base = temp_base.Replace("&matchThreads;",hitThreads.ToString());
					temp_base = temp_base.Replace("&totalPosts;",totalRes.ToString());
					temp_base = temp_base.Replace("&matchPosts;",hitRes.ToString());

					temp_base = temp_base.Replace(string.Concat(loop_key,loop,loop_key),temp.ToString());
					temp_base = temp_base.Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString());
					return TemplateFieldReplacer.ReplaceField(temp_base,"%search%",param["search"]);
				}
				else
					return HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\search.html");
			}
			else
				throw HttpException.UnauthorizedException;
		}
		static void SearchBBSPage(WebHeaderInfo info,Regex regex,string temp_loop,ref StringBuilder temp,ref int hitThreads,ref int totalRes,ref int hitRes)
		{
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			StringBuilder res = new StringBuilder();
			string summary = "";
			int hits = 0;
			foreach(WebPostInfo post in posts)
			{
				totalRes++;
				Match m = regex.Match(post.Body);
				if(m.Success)
				{
					hitRes++;
					if(hits < 4)
						summary += "..." + HttpUtility.HtmlEncode(m.Groups["summary"].Value) + "...<br />";

					res.Append(string.Concat(TextUtility.GuidToResID(post.GUID),","));
					hits++;
				}
			}
			if(hits > 0)
			{
				hitThreads++;
				string loop = TemplateFieldReplacer.ReplaceField(temp_loop,"%_summary%",summary);
				loop = TemplateFieldReplacer.ReplaceField(loop,"%_date%",info.Update.ToString("yyyy年MM月dd日 hh時mm分ss秒"));
				loop = TemplateFieldReplacer.ReplaceField(loop,"%_title%",info.Title + " [" + hits.ToString() + "レスヒット]");
				loop = TemplateFieldReplacer.ReplaceField(loop,"%_url%",string.Concat(_local,info.GUID.ToString(),"/",res.ToString().TrimEnd(',')));
				temp.Append(loop);
			}
		}
		static void SearchWikiPage(WebHeaderInfo info,Regex regex,string temp_loop,ref StringBuilder temp,ref int hitThreads,ref int totalRes,ref int hitRes)
		{
			ArrayList wikinames = new ArrayList();
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			foreach(WebPostInfo post in posts)
				if(!wikinames.Contains(post.WikiName))
					wikinames.Add(post.WikiName);
			bool incremented = false;

			foreach(string wikiname in wikinames)
			{
				totalRes++;
				WebPostInfo post = WikiGenerator.GetNewestWikiName(posts,info,wikiname,Guid.Empty);
				Match m = regex.Match(post.Body);
				if(m.Success)
				{
					hitRes++;
					string loop = TemplateFieldReplacer.ReplaceField(temp_loop,"%_summary%","..." + HttpUtility.HtmlEncode(m.Groups["summary"].Value) + "...<br />");
					loop = TemplateFieldReplacer.ReplaceField(loop,"%_date%",info.Update.ToString("yyyy年MM月dd日 hh時mm分ss秒"));
					loop = TemplateFieldReplacer.ReplaceField(loop,"%_title%",info.Title + " 「" + wikiname + "」");
					loop = TemplateFieldReplacer.ReplaceField(loop,"%_url%",string.Concat(_local,info.GUID.ToString(),"/wiki.p2p?name=",HttpUtility.UrlEncode(wikiname)));
					temp.Append(loop);

					if(!incremented)
					{
						incremented = true;
						hitThreads++;
					}
				}
			}
		}
		#endregion

		#region cache.p2p
		static string GenerateCacheToolPage()
		{
			return HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\cachetools.html");
		}
		static string ProcessCacheTool(StringDictionary param)
		{
			if(param["type"] == "restore")
			{
				if(HttpServer.HasAuthority(param))
				{
					int totalThreads = 0;
					object[] headers = Interface.GetWebHeaders();
					ArrayList brokenHeaders = new ArrayList();
					StringBuilder body = new StringBuilder();
					foreach(WebHeaderInfo header in headers)
					{
						if(WebCacheReader.IsCached(header))
						{
							bool errorFlag = false;
							try
							{
								WebPostInfo[] posts = WebCacheReader.ReadAllPost(header.GUID);
								for(int i=0;i<posts.Length;i++)
									if(posts[i] == null)
									{
										errorFlag = true;
										break;
									}
							}
							catch
							{
								errorFlag = true;
							}
							if(errorFlag)
							{
								totalThreads++;
								Guid id = header.GUID;
								body.Append("<li>" + HttpUtility.HtmlEncode(header.Title) + "：" + id.ToString() + "</li>");
								Interface.IgnorePage(id,2);
								Interface.CheckUpdatePage(id);
							}
						}
					}

					if(totalThreads > 0)
					{
						body.Insert(0,"問題が見つかりました。以下のスレッドキャッシュを削除しリクエストを発行しました。<br /><ul>");
						body.Append("</ul>");

						return HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\basic.html")
							.Replace("%TITLE%",HttpUtility.HtmlEncode("キャッシュチェックの結果"))
							.Replace("%BODY%",body.ToString());
					}
					else
					{
						return HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\basic.html")
							.Replace("%TITLE%",HttpUtility.HtmlEncode("キャッシュチェックの結果"))
							.Replace("%BODY%",HttpUtility.HtmlEncode("問題は見つかりませんでした"));
					}
				}
				else
					throw HttpException.UnauthorizedException;
			}
			else
				return GenerateCacheToolPage();
		}
		#endregion

		#region threadinfo.p2p
		static string GetThreadInfo(StringDictionary param,ref string mime)
		{
			if(param.ContainsKey("type"))
			{
				mime = "text/plain; charset=utf-8";
				return GetThreadInfo2(param);
			}
         else if(param.ContainsKey("guid"))
			{
				try
				{
					bool hasCache = true;
					Guid id = new Guid(param["guid"]);
					WebHeaderInfo info = WebCacheReader.ReadHeader(id);
					if(info == null)
					{
						hasCache = false;
						object[] headers = Interface.GetWebHeaders();
						foreach(WebHeaderInfo header in headers)
						{
							if(header.GUID == id)
							{
								info = header;
								break;
							}
						}
					}

					if(info == null)
						return "Error:該当するGUIDが見つかりません";

					StringBuilder builder = new StringBuilder();
					builder.Append("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\r\n");
					builder.Append("<html xmlns=\"http://www.w3.org/1999/xhtml\">\r\n");
					builder.Append("<head>\r\n");
					builder.Append("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\r\n");
					builder.Append("<link rel=\"stylesheet\" href=\"threadinfostyles.css\" type=\"text/css\" />");
					builder.Append("</head>\r\n");
					builder.Append("<body>\r\n");
					builder.Append(string.Concat("<span class=\"cap_type\">タイプ：</span><span id=\"type\">",HttpUtility.HtmlEncode(WebTypeToString.ToString(info.Type)),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_title\">タイトル：</span><span id=\"title\">",HttpUtility.HtmlEncode(info.Title),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_category\">カテゴリ：</span><span id=\"category\">",HttpUtility.HtmlEncode(info.Category),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_update\">更新日：</span><span id=\"update\">",HttpUtility.HtmlEncode(info.Update.ToString("yyyy\\/MM\\/dd HH\\:mm\\:ss")),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_managed\">管理更新日：</span><span id=\"managed\">",HttpUtility.HtmlEncode(info.Created.ToString("yyyy\\/MM\\/dd HH\\:mm\\:ss")),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_posts\">ポスト数：</span><span id=\"posts\">",info.Posts.ToString(),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_guid\">GUID：</span><span id=\"guid\">",HttpUtility.HtmlEncode(info.GUID.ToString()),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_size\">サイズ：</span><span id=\"size\">",TextUtility.SizeToString(info.Size),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_hash\">ハッシュ：</span><span id=\"hash\">",HttpUtility.HtmlEncode(TextUtility.ByteArrayToString(info.Hash)),"</span><br/>\r\n"));
					builder.Append(string.Concat("<span class=\"cap_cached\">状態：</span><span id=\"cache\">",(hasCache?"キャッシュ済み":"キャッシュ未取得"),"</span><br/>\r\n"));
					builder.Append("</body></html>");
					return builder.ToString();
				}
				catch
				{
					return "Error:パラメータが不正です";
				}
			}
			else
				return "Error:パラメータが入っていません";
		}
		static string GetThreadInfo2(StringDictionary param)
		{
			bool flag = (param["type"].ToLower().Equals("all"));
			StringBuilder builder = new StringBuilder();
			object[] headers = Interface.GetWebHeaders();

			foreach(WebHeaderInfo info in headers)
			{
				builder.Append(HttpUtility.HtmlEncode(info.Title));
				builder.Append("<>");
				builder.Append(info.GUID.ToString());

				if(flag)
				{
					builder.Append("<>");
					builder.Append(HttpUtility.HtmlEncode(WebTypeToString.ToString(info.Type)));
					builder.Append("<>");
					builder.Append(HttpUtility.HtmlEncode(info.Category));
					builder.Append("<>");
					builder.Append(info.Update.ToString("yyyy\\/MM\\/dd HH\\:mm\\:ss"));
					builder.Append("<>");
					builder.Append(info.Created.ToString("yyyy\\/MM\\/dd HH\\:mm\\:ss"));
					builder.Append("<>");
					builder.Append(info.Posts.ToString());
					builder.Append("<>");
					builder.Append(HttpUtility.HtmlEncode(TextUtility.SizeToString(info.Size)));
					builder.Append("<>");
					builder.Append(HttpUtility.HtmlEncode(TextUtility.ByteArrayToString(info.Hash)));
					builder.Append("<>");
					builder.Append(WebCacheReader.IsCached(info.GUID).ToString());
				}

				builder.Append("\r\n");
			}

			return builder.ToString();
		}
		#endregion
	}
}