﻿using System;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;

namespace RinGOch.HtmlGenerators
{
	/// <summary>
	/// TemplateSyntax v0.1
	/// </summary>
	public class TemplateFieldReplacer
	{
		#region 共通
		/// <summary>
		/// 共通なヘッダフィールドを置き換えします
		/// </summary>
		public static string ReplaceCommonHeaderFields(string template,WebHeaderInfo info)
		{
			if(info == null)return template;

			template = ReplaceField(template,"%_title%",info.Title);
			template = ReplaceField(template,"%_guid%",info.GUID.ToString());
			template = ReplaceField(template,"%_total_posts%",info.Posts.ToString());
			template = ReplaceField(template,"%_total_byte_size%",info.Size.ToString());
			template = ReplaceField(template,"%_total_size%",TextUtility.SizeToString(info.Size));
			template = ReplaceField(template,"%_date_update%",TextUtility.DateTimeToString(info.Update));
			template = ReplaceField(template,"%_date_update_iso%",info.Update.ToString ("yyyy'-'MM'-'dd'T'HH':'mm':'sszzz"));
			template = ReplaceField(template,"%_date_mgr_update%",TextUtility.DateTimeToString(info.Created));
			template = ReplaceField(template,"%_type%",WebTypeToString.ToString(info.Type));
			template = ReplaceField(template,"%_category%",info.Category);
			template = ReplaceField(template,"%_hash%",TextUtility.ByteArrayToString(info.Hash));

			return template;
		}

		/// <summary>
		/// 共通なポストフィールドを置き換えします
		/// </summary>
		public static string ReplaceCommonPostFields(string template,WebPostInfo post)
		{
			if(post == null)return template;

			template = ReplaceField(template,"%_post_name%",post.Name);
			template = ReplaceField(template,"%_post_date%",TextUtility.DateTimeToString(post.Posted));
			template = ReplaceField(template,"%_post_guid%",post.GUID.ToString());
			if(post.Trip != null)
				template = ReplaceField(template,"%_post_short_trip%",TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip)));
			else
				template = ReplaceField(template,"%_post_short_trip%","");
			template = ReplaceField(template,"%_post_filename%",post.FileName);
			if(post.Data != null)
				template = ReplaceField(template,"%_post_filesize%",TextUtility.SizeToString(post.Data.Length));
			template = ReplaceField(template,"%_post_body%",post.Body);
			return template;
		}

		/// <summary>
		/// 関数を適用した置き換えを実施します
		/// </summary>
		public static string ReplaceField(string template,string field,string newText)
		{
			return ReplaceField(template,field,newText,HttpServerSetting.HtmlEncoding);
		}
		public static string ReplaceField(string template,string field,string newText,Encoding encoding)
		{
			return template.Replace("&" + field,HttpUtility.UrlEncode(newText,encoding))
				.Replace("<" + field,HttpUtility.HtmlEncode(newText)).Replace(field,newText);
		}
		#endregion

		#region Wiki
		/// <summary>
		/// Wikiの置き換え
		/// </summary>
		public static string ReplaceWikiFields(string template,WebHeaderInfo info,WebPostInfo post,WebPostInfo[] posts)
		{
			return ReplaceWikiFields(template,info,post,posts,false);
		}
		public static string ReplaceWikiFields(string template,WebHeaderInfo info,WebPostInfo post,WebPostInfo[] posts,bool checkSign)
		{
			return ReplaceWikiFields(template,info,post,posts,ref checkSign);
		}
		public static string ReplaceWikiFields(string template,WebHeaderInfo info,WebPostInfo post,WebPostInfo[] posts,ref bool checkSign)
		{
			string temp = ReplaceCommonHeaderFields(template,info);
			if(posts != null) temp = ReplaceField(temp,"%_wiki_menubar%",GetWikiMenubar(info,post,posts));
			if(post != null)
			{
				string nametrip = "";
				if(post.Trip != null)
				{
					if(!checkSign || (checkSign && post.IsEffectiveSignature()))
					{
						nametrip = post.Name.Replace("◆","") + "◆" + TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip));

						const string freezeText = "(ロック中)";
						if(ByteArrayComparer.Comparer(post.Trip,info.Trip)==0)
							nametrip += freezeText;

						checkSign = false;
					}
					else
						nametrip = post.Name.Replace("◆","") + "◆認証失敗：" + TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip));
				}
				else
					nametrip = post.Name.Replace("◆","");

				temp = ReplaceField(temp,"%_wiki_hierarchical_link%",CreateWikiHierarchicalLink(info,post));
				temp = ReplaceField(temp,"%_wikiname%",post.WikiName);
				temp = ReplaceField(temp,"%_wiki_nametrip%",nametrip);
				temp = ReplaceCommonPostFields(temp,post);
				return temp;
			}
			else
				return temp;
		}
		static string CreateWikiHierarchicalLink(WebHeaderInfo info,WebPostInfo post)
		{
			string wikiname = post.WikiName;
			StringBuilder link = new StringBuilder();
			if(post.FileName.StartsWith(info.Title + "/"))
			{
				post.FileName = post.FileName.Substring(info.Title.Length+1);
				link.Append("<a href=\"wiki.p2p\">" + info.Title + "</a>/");
			}

			string[] names = wikiname.Split('/');
			string log = "";
			foreach(string name in names)
			{
				link.Append("<a href=\"wiki.p2p?name=" + log + HttpUtility.UrlEncode(name) + "\">" + HttpUtility.HtmlEncode(name) + "</a>/");
				log += HttpUtility.UrlEncode(name + "/");
			}

			if(link.Length > 0)
				link.Remove(link.Length-1,1);
			else
				link.Append(post.WikiName);

			return link.ToString();
		}
		public static string GetWikiMenubar(WebHeaderInfo info,WebPostInfo post,WebPostInfo[] posts)
		{
			WebPostInfo menubar = WikiGenerator.GetNewestWikiName(posts,info,"MenuBar",Guid.Empty);
			if(menubar == null) return "";
			StringBuilder html = new StringBuilder();
			html.Append("<div class=\"menubar\">");

			html.Append(WikiEngine.Engine.Transform(menubar.Body,info,posts,menubar));

			html.Append("</div>");
			return html.ToString();
		}
		#endregion

		#region Gateway
		public static string ReplaceGatewayFields(string template,WebHeaderInfo info,string pageStatus,int pageIndex)
		{
			string temp = template;
			
			temp = temp.Replace("%_gateway_mod2%",(pageIndex % 2).ToString());

			if(info != null)
			{
				temp = ReplaceField(temp,"%_gateway_pagestatus%",pageStatus);
				temp = ReplaceCommonHeaderFields(temp,info);
			}

			return ReplaceField(temp,"%_gateway_pageindex%",pageIndex.ToString());
		}
		#endregion

		#region BBS
		public static string ReplaceBbsFields(string template,WebHeaderInfo info,WebPostInfo post,bool signOK,string thumUri,int postIndex,string uploader,string tripList)
		{
			string temp = template;

			if(info != null)
			{
				if(uploader != "")
				{
					if(info.Type == WebType.ImageBoard || info.Type == WebType.FileBoard)
						temp = ReplaceField(temp,"%_bbs_uploader%",uploader);
					else
						temp = ReplaceField(temp,"%_bbs_uploader%","");
				}
				if(tripList != "")
					temp = ReplaceField(temp,"%_bbs_triplist%",tripList);

				temp = ReplaceCommonHeaderFields(temp,info);
			}

			if(post != null)
			{
				string trip = "";
				if(post.Trip != null)
				{
					if(signOK)
						trip = "<span class=\"signOK\">◆" + TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip)) + "</span>";
					else
						trip = "<span class=\"signError\">◆認証エラー (" + TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip)) + ")</span>";
				}
				temp = ReplaceField(temp,"%_bbs_short_guid%",TextUtility.GuidToResID(post.GUID));
				temp = ReplaceField(temp,"%_bbs_thumbnail%",thumUri);
				temp = ReplaceField(temp,"%_bbs_post_index%",postIndex.ToString());
				if(post.Trip != null)
				{
					if(signOK)
						temp = temp.Replace("%_bbs_short_trip%","<span class=\"signOK\">◆" + TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip)) + "</span>");
					else
						temp = temp.Replace("%_bbs_short_trip%","<span class=\"signError\">◆認証エラー (" + TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip)) + ")</span>");
				}
				else
					temp = temp.Replace("%_bbs_short_trip%","");
				string name = post.Name;
				string body = post.Body;
				post.Name = post.Name.Replace("◆","");

				// WikiEngine for BBS (凍結)
				//if(post.Body.StartsWith("@wiki\r\n"))
				//{
				//	WikiEngine.ContentsTransformer transformer = new RinGOch.WikiEngine.ContentsTransformer(info,new WebPostInfo[0],post,"_blank");
				//	transformer.replacer.enableEmbedded = false;
				//	transformer.replacer.BlogMode = false;
				//	transformer.replacer.enableStyleAttribute = false;
				//	post.Body = transformer.Transform(post.Body.Substring(5));
				//}
				//else
				//{
				post.Body = HttpUtility.HtmlEncode(post.Body);
				post.Body = HttpHtmlGenerator.CreateURLLink(post.Body,true);
				post.Body = post.Body.Replace("\r\n","\n").Replace("\r","\n").Replace("\n","<br>");
				//}
				temp = ReplaceCommonPostFields(temp,post);
				post.Name = name;
				post.Body = body;
			}

			if(HttpServerSetting.PublicMode)
				temp = ReplaceField(temp,"%_bbs_dir_thumb%","");
			else
				temp = ReplaceField(temp,"%_bbs_dir_thumb%",BBSGenerator.ThumbnailDirectory.Replace("\\","/"));

			return temp.ToString();
		}
		#endregion

		#region 管理
		public static string GenerateCategoryList(string selected)
		{
			StringBuilder sb = new StringBuilder();
			foreach(string cate in WebCategories.Categories)
			{
				sb.Append("<option" + (cate==selected?" selected":"") + ">" + cate + "</option>");
			}
			return sb.ToString();
		}
		#endregion
	}
}
