﻿using System;
using System.Text;
using System.Collections.Specialized;
using System.Threading;
using System.IO;
using System.Net;
using System.Net.Sockets;

namespace RinGOch
{
	public class HttpServerLogger
	{
		static AutoResetEvent _done = new AutoResetEvent(true);

		// ログを追加します
		public static void AppendLog(Socket sock,StringDictionary param,string msg2,int dataSize)
		{
			int ret = 500;
			string msg = msg2.Trim();
			if(msg == "") ret = 500;
			if(msg.Length > 3 && TextUtility.IsNumeric(msg.Substring(0,3)))
				ret = int.Parse(msg.Substring(0,3));

			AppendLog(sock,param,ret,dataSize);
		}
		public static void AppendLog(Socket sock,StringDictionary param,int retValue,int dataSize,Exception ex)
		{
			const string space = " ";

			// ログ文字列生成
			StringBuilder buffer = new StringBuilder();

			// ホスト追加
			try
			{
				buffer.Append(((IPEndPoint)sock.RemoteEndPoint).Address.ToString());
			}
			catch
			{
				buffer.Append("UnknownIP");
			}
			buffer.Append(space);
			buffer.Append("- -");
			buffer.Append(space);

			// 日付追加
			buffer.Append("[");
			buffer.Append(DateTime.Now.ToString("dd\\/MMM\\/yyyy\\:HH\\:mm\\:ss",System.Globalization.DateTimeFormatInfo.InvariantInfo));
			buffer.Append(" +0900]");
			buffer.Append(space);

			// パラメータ追加
			buffer.Append("\"");
			try
			{
				buffer.Append(param["_method"].TrimEnd("\r\"".ToCharArray()));
			}
			catch
			{
				buffer.Append("Unknown");
			}
			buffer.Append(space);

			try
			{
				buffer.Append(param["_path"].TrimEnd("\r\"".ToCharArray()));
			}
			catch
			{
				buffer.Append("Unknown");
			}
			buffer.Append(space);

			try
			{
				buffer.Append(param["_version"].TrimEnd("\r\"".ToCharArray()));
			}
			catch
			{
				buffer.Append("Unknown");
			}
			buffer.Append(space);

			buffer.Append("\"");
			buffer.Append(space);

			// 応答コード追加
			buffer.Append(retValue.ToString());
			buffer.Append(space);

			// 送信サイズを追加
			buffer.Append(dataSize.ToString());
			buffer.Append(space);

			// リファラ追加
			if(param.ContainsKey("_referer"))
				buffer.Append("\"" + param["_referer"] + "\"");
			else
				buffer.Append("\"\"");
			buffer.Append(space);

			// ユーザーえいじぇんと追加
			if(param.ContainsKey("_user-agent"))
				buffer.Append("\"" + param["_user-agent"] + " " + ex.Message.Replace("\n","").Replace("\r","") + "\"");
			else
				buffer.Append("\"" + " " + ex.Message.Replace("\n","").Replace("\r","") + "\"");

			// 改行追加
			buffer.Append("\r\n");

			AppendLog(buffer.ToString());
		}
		public static void AppendLog(Socket sock,StringDictionary param,int retValue,int dataSize)
		{
			const string space = " ";

			// ログ文字列生成
			StringBuilder buffer = new StringBuilder();

			// ホスト追加
			try
			{
				buffer.Append(((IPEndPoint)sock.RemoteEndPoint).Address.ToString());
			}
			catch
			{
				buffer.Append("UnknownIP");
			}
			buffer.Append(space);
			buffer.Append("- -");
			buffer.Append(space);

			// 日付追加
			buffer.Append("[");
			buffer.Append(DateTime.Now.ToString("dd\\/MMM\\/yyyy\\:HH\\:mm\\:ss",System.Globalization.DateTimeFormatInfo.InvariantInfo));
			buffer.Append(" +0900]");
			buffer.Append(space);

			// パラメータ追加
			buffer.Append("\"");
			try
			{
				buffer.Append(param["_method"].TrimEnd("\r\"".ToCharArray()));
			}
			catch
			{
				buffer.Append("Unknown");
			}
			buffer.Append(space);

			try
			{
				buffer.Append(param["_path"].TrimEnd("\r\"".ToCharArray()));
			}
			catch
			{
				buffer.Append("Unknown");
			}
			buffer.Append(space);

			try
			{
				buffer.Append(param["_version"].TrimEnd("\r\"".ToCharArray()));
			}
			catch
			{
				buffer.Append("Unknown");
			}
			buffer.Append(space);

			buffer.Append("\"");
			buffer.Append(space);

			// 応答コード追加
			buffer.Append(retValue.ToString());
			buffer.Append(space);

			// 送信サイズを追加
			buffer.Append(dataSize.ToString());
			buffer.Append(space);

			// リファラ追加
			if(param.ContainsKey("_referer"))
				buffer.Append("\"" + param["_referer"] + "\"");
			else
				buffer.Append("\"\"");
			buffer.Append(space);

			// ユーザーえいじぇんと追加
			if(param.ContainsKey("_user-agent"))
				buffer.Append("\"" + param["_user-agent"] + "\"");
			else
				buffer.Append("\"\"");

			// 改行追加
			buffer.Append("\r\n");

			AppendLog(buffer.ToString());
		}
		static void AppendLog(string line)
		{
			try
			{
				_done.WaitOne();
				string path = Path.Combine(HttpServerSetting.LogFilePath,DateTime.Now.ToString("yyyyMMdd")) + ".log";
				if(!File.Exists(path)) File.Create(path).Close();
				using(FileStream fs = new FileStream(path,FileMode.Append,FileAccess.Write,FileShare.Read))
				{
					byte[] data = System.Text.Encoding.Default.GetBytes(line);
					fs.Write(data,0,data.Length);
					fs.Close();
				}
			}
			catch(Exception ex)
			{
				Console.WriteLine(ex.ToString());
			}
			finally
			{
				_done.Set();
			}
		}
	}
}
