﻿using System;

namespace RinGOch
{
	public class Interface
	{
		public delegate bool CanPostDelegate();
		public delegate object[] GetWebHeadersDelegate();
		public delegate bool CanCreatePageDelegate(out int remain);
		public delegate bool IsCachedPageDelegate(Guid id);
		public delegate System.Drawing.Bitmap GenerateImageDelegate();
		public delegate short CheckUpdateDelegate(Guid id);
		public delegate Guid PostToBbsDelegate(Guid page,string name,string body,string trip,byte[] binary,string filename);
		public delegate bool IsEffectiveCertificationKeyDelegate(string key);
		public delegate void CreatePageDelegate(string title,string cate,int type,string certificationKey,out Guid pageid);
		public delegate string[] GetTripListDelegate();
		public delegate Guid[] GetManagePagesDelegate();
		public delegate string ManagePageDelegate(Guid pageid,string title,string cate,Guid[] deletePosts);
		public delegate object ReadCachedWebHeaderDelegate(Guid id);
		public delegate void GetInformationDelegate(out string version,out int connections);
		public delegate void IgnoreDelegate(Guid id,int type);
		public delegate Guid[] GetIgnoreListDelegate();
		public delegate string[] ChatReadMessageDelegate(string room,int count);
		public delegate bool ChatPostMessageDelegate(string room,string msg,string iconPath,System.Net.IPAddress client);
		public delegate void WebCacheDelegate(Guid id);
		public delegate byte[] ChatIconDelegate(byte[] hash);
		public delegate string[] ChatGetIconFilesDelegate();

		#region P2PWeb

		#region 問い合わせ

		/// <summary>
		/// 投稿可能かどうか判断します。（投稿制限）
		/// </summary>
		public static CanPostDelegate CanPost = null;

		/// <summary>
		/// 各種情報を取得します
		/// </summary>
		public static GetInformationDelegate GetInformation = null;

		/// <summary>
		/// スレ立てが出来るか判断します。(スレ立て制限)
		/// </summary>
		public static CanCreatePageDelegate CanCreatePage = null;
		public static bool CanCreatePageB()
		{
			int remain;
			return CanCreatePage(out remain);
		}


		#endregion

		#region キャッシュ

		/// <summary>
		/// キャッシュファイルをロック/アンロックします。
		/// ロックしっぱなしだと動作不良を起こすので、必ずアンロックしてくださいね
		/// </summary>
		public static WebCacheDelegate LockWebCache = null;
		public static WebCacheDelegate UnlockWebCache = null;

		/// <summary>
		/// 指定されたGUIDを持つページがキャッシュされているか調べます
		/// </summary>
		public static IsCachedPageDelegate IsCachedPage = null;

		/// <summary>
		/// キャッシュ済みのWebHeaderを読み込みます。
		/// </summary>
		public static ReadCachedWebHeaderDelegate ReadCachedWebHeader = null;

		/// <summary>
		/// Webキャッシュを削除します
		/// </summary>
		public static WebCacheDelegate DeleteWebCache = null;

		#endregion

		/// <summary>
		/// Webページ一覧（スレ一覧）を取得します。
		/// 戻り値は、WebHeaderInfo[]です
		/// </summary>
		public static GetWebHeadersDelegate GetWebHeaders = null;

		/// <summary>
		/// Webスレ立て用認証画像を取得します。
		/// </summary>
		public static GenerateImageDelegate GenerateCertificationImage = null;

		/// <summary>
		/// Webページが更新されているか確認するメッセージを発行します。
		/// なお、悪戯防止策としてゆるいリミッターつきです。
		/// 諸事情により、戻り値がshortになってしまいましたが、戻り値は意味ありません。
		/// </summary>
		public static CheckUpdateDelegate CheckUpdatePage = null;

		/// <summary>
		/// BBSへ投稿します。投稿前にCanPostで制限中じゃないか確認することをおすすめします。
		/// trueなら書き込み成功。falseならエラーです。
		/// </summary>
		public static PostToBbsDelegate PostToBbs = null;

		/// <summary>
		/// スレ立てのための認証文字列が有効かチェックします。
		/// </summary>
		public static IsEffectiveCertificationKeyDelegate IsEffectiveCertificationKey = null;

		/// <summary>
		/// スレ立てします。投稿前にIsEffectiveCertificationKey、CanCreatePageを使ってスレ立て可能な常態か確認してください
		/// </summary>
		public static CreatePageDelegate CreatePage = null;

		/// <summary>
		/// トリップ一覧を取得します。（すでに表示用エンコード済み）
		/// </summary>
		public static GetTripListDelegate GetTripList = null;

		/// <summary>
		/// 管理ページGUID一覧を取得します
		/// </summary>
		public static GetManagePagesDelegate GetManagePages = null;

		/// <summary>
		/// 管理更新します。
		/// </summary>
		public static ManagePageDelegate ManagePage = null;

		/// <summary>
		/// スレを無視等を管理します。第二引数が０なら無視+キャッシュ削除。１なら無視解除。２ならキャッシュ削除です
		/// </summary>
		public static IgnoreDelegate IgnorePage = null;

		#endregion

		#region P2PChat

		/// <summary>
		/// チャットメッセージを読み取ります。countには最大メッセージ数を指定
		/// </summary>
		public static ChatReadMessageDelegate ChatReadMessage = null;

		/// <summary>
		/// チャットにメッセージを投稿します
		/// </summary>
		public static ChatPostMessageDelegate ChatPostMessage = null;

		/// <summary>
		/// チャットアイコンハッシュからアイコンのバイナリデータを読み込みます
		/// </summary>
		public static ChatIconDelegate ChatGetIconBinary = null;

		/// <summary>
		/// チャットアイコンのファイル名を返します。フルパスの場合もあるので（ぉぃ）RinGOch側でフォーマットしてください
		/// </summary>
		public static ChatGetIconFilesDelegate ChatGetIcons = null;

		#endregion
	}
}
