﻿using System;
using System.Net;

namespace RinGOch
{
	public class NetworkAddress
	{
		static IPAddress[] _adrs;
		static DateTime	_lastCheck;

		static NetworkAddress()
		{
			UpdateLocalAddress();
		}

		static void UpdateLocalAddress()
		{
			_adrs = Dns.Resolve(Dns.GetHostName()).AddressList;
			_lastCheck = DateTime.Now;
		}

		public static IPAddress LocalAddress
		{
			get
			{
				IPAddress[] adrs = _adrs;

				for(int i=0;i<adrs.Length;i++)
					if(!IsPrivateAddress(adrs[i]))
						return adrs[i];

				if(DateTime.Now.Subtract(_lastCheck).TotalMinutes > 10)
				{
					new System.Threading.ThreadStart(UpdateLocalAddress).BeginInvoke(null,null);
				}

				return adrs[0];

			}
		}

		public static bool IsPrivateAddress(IPAddress adrs)
		{
			byte[] data = adrs.GetAddressBytes();

			if(data.Length == 4)
			{
				// IPv4 Network Address
				if(data[0]==192 && data[1]==168)
					return true;
				if(data[0]==172 && (data[1]==16 | data[1]==31))
					return true;
				if(data[0]==10)
					return true;
				if(data[0]==169)
					return true;

				return false;
			}
			else
				return false;  // IPv6 Network Address
		}

		public static IPAddress IPAddressParse(byte[] data)
		{
			if(data.Length == 4)
			{
				return IPAddress.Parse(data[0] + "." + data[1] + "." + data[2] + "." + data[3]);
			}
			else if(data.Length == 16)
			{
				return new IPAddress(data);
			}
			else
				throw new Exception();
		}
	}
}
