using System;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using RinGOch.WikiEngine.LineData;
using RinGOch.WikiEngine.LineData.Chains;

namespace RinGOch.WikiEngine
{
	public class ContentsTransformer : IWikiTransformer
	{
		public InlineReplacer replacer;

		public ContentsTransformer(WebHeaderInfo header,WebPostInfo[] posts,WebPostInfo disp,string linkTarget)
		{
			replacer = new InlineReplacer(linkTarget,header,posts,disp);
		}

		public string Transform(string contents)
		{
			try
			{
				StringBuilder body = new StringBuilder();
				StringBuilder headings = new StringBuilder();

				contents = ReplaceSpecialChars(contents);
				contents = CodeTagReplacer.Transform(contents);
				contents = BlockStyleCreator.Create(contents);
				
				LineDataFactory factory = new LineDataFactory();
				ILineData previousLine = factory.Create(null);

				using(StringReader reader = new StringReader(contents))
				{
					string line = null;
					bool emptySkip = false;
					while ((line = reader.ReadLine()) != null)
					{
						ILineData nextLine = factory.Create(line);

						if(!emptySkip)
							body.Append(previousLine.GetCloseTag(nextLine));

						string c = replacer.ReplaceInline(nextLine, InlineReplacer.ReplaceType.Contents);
						if(c.Length > 0 || nextLine.Keyword=="----")
						{
							body.Append(nextLine.GetOpenTag(previousLine));
							body.Append(c);
							emptySkip = false;
						}
						else
							emptySkip = true;

						headings.Append(nextLine.GetHeading());

						previousLine = nextLine;
					}

					body.Append(previousLine.GetCloseTag(factory.Create(null)));

					return body.ToString();
				}
			}
			catch (Exception ex)
			{
				throw new Exception("eHTMLϊɎs܂B", ex);
			}
			finally
			{
				replacer.ClearState();
			}
		}

		/// <summary>
		/// HTMLꕶu܂B
		/// </summary>
		/// <param name="input">u镶</param>
		/// <returns>u̕</returns>
		public static string ReplaceSpecialChars(string input)
		{
			//string replaced = input;
			//replaced = Regex.Replace(replaced, "&", "&amp;");
			//replaced = Regex.Replace(replaced, "\"", "&quot;");
			//replaced = Regex.Replace(replaced, "<", "&lt;");
			//return Regex.Replace(replaced,"(?<keyword>^(>{1,3}))(?<content>.*)",new MatchEvaluator(KeyReplacer),RegexOptions.Multiline);
			return input.Replace("&","&amp;").Replace("\"","&quot;").Replace("<","&lt;").Replace(">","&gt;");
		}
		string KeyReplacer(Match m)
		{
			return m.Groups["keyword"].Value + m.Groups["content"].Value.Replace(">","&gt;");
		}
	}
}
